==> org-lsst-ccs-config-definition-5.3.15-2-sources.jar: org/lsst/ccs/config/ConfigurationParameterHandler.java <==
package org.lsst.ccs.config;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import org.lsst.ccs.bus.data.ConfigurationParameterType;
import org.lsst.ccs.commons.annotations.ConfigurationParameter;
import org.lsst.ccs.utilities.constraints.Constraints;
import org.lsst.ccs.utilities.conv.InputConversionEngine;
import org.lsst.ccs.utilities.conv.TypeConversionException;
import org.lsst.ccs.utilities.conv.TypeUtils;
import org.lsst.ccs.utilities.structs.ParameterPath;
import org.lsst.ccs.utilities.structs.ViewValue;

/**
 *
 * @author LSST CCS Team.
 */
public class ConfigurationParameterHandler implements Comparable<ConfigurationParameterHandler>{
     
    private final ParameterPath parameterPath;
    
    /** The annotation associated to this parameter. Cannot be null */
    private final ConfigurationParameter annotation;
    
    /** The @ConfigurationParameterChanger annotated methods. Can be null */
    private final Method parameterChangerMethod;
    
    /** The parameter itself. Cannot be null */
    private final Field parameterField;
    
    private final Object target;
    
    private final ConfigurationParameterType configType;
    
    ConfigurationParameterHandler(String componentName, Method m, Field f, Object target) {
        this.parameterChangerMethod = m;
        this.parameterField = f;
        this.target = target;
        parameterField.setAccessible(true);
        annotation = f.getAnnotation(ConfigurationParameter.class);
        checkAgainstConstraints(getObjectValue());
        this.parameterPath = new ParameterPath(componentName, (annotation.name().isEmpty()) ? f.getName() : annotation.name());
        this.configType = annotation.isFinal() ? ConfigurationParameterType.FINAL : 
                annotation.isBuild() ? ConfigurationParameterType.BUILD :
                annotation.isReadOnly() ? ConfigurationParameterType.READ_ONLY :
                ConfigurationParameterType.RUNTIME;
    }

    /**
     * Single setting of a parameter. It first attempts to find the
     * corresponding @ParameterChanger annotated method. If not found, it 
     * attempts to set the corresponding @Parameter annotated field.
     *
     * @param parameterName the parameter
     * @param arg the new value for the parameter
     */
    void invokeSetParameter(Object arg) {
            Method m = parameterChangerMethod;
            try {
                if (m != null) {
                    m.setAccessible(true);
                    m.invoke(target, arg);
                } else {
                    Field f = parameterField;
                    f.setAccessible(true);
                    f.set(target, arg);
                }
            } catch (IllegalAccessException | InvocationTargetException ex ) {
                Throwable cause = ex.getCause();
                String msg = cause == null ? ex.toString() : ex.getCause().getMessage();
                throw new RuntimeException("at parameter " + parameterPath + " : " + msg, ex);
            }
    }
    
    public ParameterPath getParameterPath() {
        return parameterPath;
    }
    
    String getCategory() {
        return annotation.category();
    }
    
    public boolean isFinal() {
        return configType == ConfigurationParameterType.FINAL;
    }

    public boolean isReadOnly() {
        return configType == ConfigurationParameterType.READ_ONLY;
    }

    public boolean isBuild() {
        return configType == ConfigurationParameterType.BUILD;
    }
    
    public ConfigurationParameterType getParameterType() {
        return configType;
    }
    
    String getUnits() {
        return annotation.units();
    }
    
    String getDescription() {
        return annotation.description();
    }
    
    String getRange() {
        return annotation.range();
    }
    
    final void checkAgainstConstraints(Object val) {
        String range = annotation.range();
        if (range != null) {
            Constraints.check(val, range);
        }
    }
        
    String getParameterName() {
        return parameterPath.getParameterName();
    }
    
    String getComponentName() {
        return parameterPath.getComponentName();
    }
    
    Type getType() { 
        return parameterField.getGenericType();
    }
    
    final String getValue() {
        return TypeUtils.stringify(getObjectValue());
    }
    
    private Object getObjectValue() {
        Object res;
        try {
            parameterField.setAccessible(true);
            res = parameterField.get(target);
            parameterField.setAccessible(false);
        } catch (IllegalAccessException ex) {
            throw new RuntimeException("could not access " + parameterField.getName(), ex);
        }
        return res;
    }
    
    ViewValue convert(String strValue) {
        try {
            Object val = InputConversionEngine.convertArgToType(strValue,
                    parameterField.getGenericType());
            return new ViewValue(strValue, val);
        } catch (TypeConversionException ex) {
            throw new IllegalArgumentException(
                    "failure converting : "+ parameterPath.toString()
                            + " with value "+ strValue
                            + " to type "+ parameterField.getGenericType().getTypeName(),
                    ex);
        }
    }

    @Override
    public int compareTo(ConfigurationParameterHandler o) {
        return this.getParameterPath().compareTo(o.getParameterPath());
    }
    


}

==> org-lsst-ccs-config-definition-5.3.19-sources.jar: org/lsst/ccs/config/ConfigurationParameterHandler.java <==
package org.lsst.ccs.config;

import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.Type;
import java.util.Collection;
import java.util.Map;
import org.lsst.ccs.bus.data.ConfigurationParameterType;
import org.lsst.ccs.commons.annotations.ConfigurationParameter;
import org.lsst.ccs.utilities.constraints.Constraints;
import org.lsst.ccs.utilities.conv.InputConversionEngine;
import org.lsst.ccs.utilities.conv.TypeConversionException;
import org.lsst.ccs.utilities.conv.TypeUtils;
import org.lsst.ccs.utilities.structs.ParameterPath;
import org.lsst.ccs.utilities.structs.ViewValue;

/**
 *
 * @author LSST CCS Team.
 */
public class ConfigurationParameterHandler implements Comparable<ConfigurationParameterHandler>{
     
    private final ParameterPath parameterPath;
    
    /** The annotation associated to this parameter. Cannot be null */
    private final ConfigurationParameter annotation;
    
    /** The @ConfigurationParameterChanger annotated methods. Can be null */
    private final Method parameterChangerMethod;
    
    /** The parameter itself. Cannot be null */
    private final Field parameterField;
    
    private final Object target;
    
    private final ConfigurationParameterType configType;
    
    private final boolean hasLength;
    
    private String externalCategory = null;
    
    ConfigurationParameterHandler(String componentName, Method m, Field f, Object target) {
        this.parameterChangerMethod = m;
        this.parameterField = f;
        this.target = target;
        parameterField.setAccessible(true);
        annotation = f.getAnnotation(ConfigurationParameter.class);
        this.parameterPath = new ParameterPath(componentName, (annotation.name().isEmpty()) ? f.getName() : annotation.name());
        this.configType = annotation.isFinal() ? ConfigurationParameterType.FINAL : 
                annotation.isBuild() ? ConfigurationParameterType.BUILD :
                annotation.isReadOnly() ? ConfigurationParameterType.READ_ONLY :
                ConfigurationParameterType.RUNTIME;        

        Class type = parameterField.getType();
        hasLength = type.isArray() || Collection.class.isAssignableFrom(type) || Map.class.isAssignableFrom(type);
    }

    /**
     * Single setting of a parameter. It first attempts to find the
     * corresponding @ParameterChanger annotated method. If not found, it 
     * attempts to set the corresponding @Parameter annotated field.
     *
     * @param parameterName the parameter
     * @param arg the new value for the parameter
     */
    void invokeSetParameter(Object arg) {
            Method m = parameterChangerMethod;
            try {
                if (m != null) {
                    m.setAccessible(true);
                    m.invoke(target, arg);
                } else {
                    Field f = parameterField;
                    f.setAccessible(true);
                    f.set(target, arg);
                }
            } catch (IllegalAccessException | InvocationTargetException ex ) {
                Throwable cause = ex.getCause();
                String msg = cause == null ? ex.toString() : ex.getCause().getMessage();
                throw new RuntimeException("at parameter " + parameterPath + " : " + msg, ex);
            }
    }
    
    void acceptValue(Object obj) {
        if ( hasLength && getMaxLength() > 0 ) {
            Class type = parameterField.getType();
            int objectSize = -1;
            if ( type.isArray() ) {
                objectSize = Array.getLength(obj);
            } else if (Collection.class.isAssignableFrom(type)) {
                objectSize = ((Collection)obj).size();
            } else if (Map.class.isAssignableFrom(type)) {
                objectSize = ((Map)obj).size();
            }
            if ( objectSize > getMaxLength() ) {
                throw new RuntimeException("Size of committed value ("+objectSize+") exceeds the maximum length:"+getMaxLength());
            }
        }
    }
    
    public ParameterPath getParameterPath() {
        return parameterPath;
    }
    
    public String getCategory() {        
        return externalCategory != null ? externalCategory : annotation.category();
    }
    
    void setCategory(String category) {
        this.externalCategory = category;
    }
    
    public boolean hasConfigChangerMethod() {
        return parameterChangerMethod != null;
    }
    public Method getConfigChangerMethod() {
        return parameterChangerMethod;
    }
    
    public boolean isFinal() {
        return configType == ConfigurationParameterType.FINAL;
    }

    public boolean isReadOnly() {
        return configType == ConfigurationParameterType.READ_ONLY;
    }

    public boolean isBuild() {
        return configType == ConfigurationParameterType.BUILD;
    }
    
    public ConfigurationParameterType getParameterType() {
        return configType;
    }
    
    public boolean isOptional() {
        return annotation.isOptional();
    }
    
    public String getUnits() {
        return annotation.units();
    }
    
    String getDescription() {
        return annotation.description();
    }
    
    String getRange() {
        return annotation.range();
    }
    
    public final void checkAgainstConstraints(Object val) {
        String range = annotation.range();
        if (range != null) {
            Constraints.check(val, range);
        }
    }
        
    String getParameterName() {
        return parameterPath.getParameterName();
    }
    
    String getComponentName() {
        return parameterPath.getComponentName();
    }
    
    Type getType() { 
        return parameterField.getGenericType();
    }
    
    final Field getField() {
        return parameterField;
    }
    
    final int getMaxLength() {
        return annotation.maxLength();
    }
    
    final boolean hasLength() {
        return hasLength;
    }
    
    final String getValue() {
        return TypeUtils.stringify(getObjectValue());
    }
    
    private Object getObjectValue() {
        Object res;
        try {
            parameterField.setAccessible(true);
            res = parameterField.get(target);
            parameterField.setAccessible(false);
        } catch (IllegalAccessException ex) {
            throw new RuntimeException("could not access " + parameterField.getName(), ex);
        }
        return res;
    }
    
    ViewValue convert(String strValue) {
        try {
            Object val = InputConversionEngine.convertArgToType(strValue,
                    parameterField.getGenericType());
            return new ViewValue(strValue, val);
        } catch (TypeConversionException ex) {
            throw new IllegalArgumentException(
                    "failure converting : "+ parameterPath.toString()
                            + " with value "+ strValue
                            + " to type "+ parameterField.getGenericType().getTypeName(),
                    ex);
        }
    }

    @Override
    public int compareTo(ConfigurationParameterHandler o) {
        return this.getParameterPath().compareTo(o.getParameterPath());
    }
    


}
